@@ -1261,7 +1261,18 @@ char *get_map_shm(struct sock_info *sock_info)
 		lttng_ust_unlock_fd_tracker();
 		goto error;
 	}
-	lttng_ust_add_fd_to_tracker(wait_shm_fd);
+
+	ret = lttng_ust_add_fd_to_tracker(wait_shm_fd);
+	if (ret < 0) {
+		ret = close(wait_shm_fd);
+		if (!ret) {
+			PERROR("Error closing fd");
+		}
+		lttng_ust_unlock_fd_tracker();
+		goto error;
+	}
+
+	wait_shm_fd = ret;
 	lttng_ust_unlock_fd_tracker();
 
 	wait_shm_mmap = mmap(NULL, page_size, PROT_READ,
@@ -1353,7 +1364,7 @@ static
 void *ust_listener_thread(void *arg)
 {
 	struct sock_info *sock_info = arg;
-	int sock, ret, prev_connect_failed = 0, has_waited = 0;
+	int sock, ret, prev_connect_failed = 0, has_waited = 0, fd;
 	long timeout;
 
 	lttng_ust_fixup_tls();
@@ -1385,6 +1396,10 @@ restart:
 		prev_connect_failed = 0;
 	}
 
+	if (ust_lock()) {
+		goto quit;
+	}
+
 	if (sock_info->socket != -1) {
 		/* FD tracker is updated by ustcomm_close_unix_sock() */
 		ret = ustcomm_close_unix_sock(sock_info->socket);
@@ -1404,9 +1419,6 @@ restart:
 		sock_info->notify_socket = -1;
 	}
 
-	if (ust_lock()) {
-		goto quit;
-	}
 
 	/*
 	 * Register. We need to perform both connect and sending
@@ -1433,9 +1445,21 @@ restart:
 		ust_unlock();
 		goto restart;
 	}
-	lttng_ust_add_fd_to_tracker(ret);
-	lttng_ust_unlock_fd_tracker();
+	fd = ret;
+	ret = lttng_ust_add_fd_to_tracker(fd);
+	if (ret < 0) {
+		ret = close(fd);
+		if (ret) {
+			PERROR("close on sock_info->socket");
+		}
+		ret = -1;
+		lttng_ust_unlock_fd_tracker();
+		ust_unlock();
+		goto quit;
+	}
+
 	sock_info->socket = ret;
+	lttng_ust_unlock_fd_tracker();
 
 	ust_unlock();
 	/*
@@ -1504,9 +1528,22 @@ restart:
 		ust_unlock();
 		goto restart;
 	}
-	lttng_ust_add_fd_to_tracker(ret);
-	lttng_ust_unlock_fd_tracker();
+
+	fd = ret;
+	ret = lttng_ust_add_fd_to_tracker(fd);
+	if (ret < 0) {
+		ret = close(fd);
+		if (ret) {
+			PERROR("close on sock_info->notify_socket");
+		}
+		ret = -1;
+		lttng_ust_unlock_fd_tracker();
+		ust_unlock();
+		goto quit;
+	}
+
 	sock_info->notify_socket = ret;
+	lttng_ust_unlock_fd_tracker();
 
 	ust_unlock();
 	/*
@@ -1949,6 +1986,7 @@ void ust_after_fork_child(sigset_t *restore_sigset)
 {
 	if (URCU_TLS(lttng_ust_nest_count))
 		return;
+	lttng_context_vpid_reset();
 	lttng_context_vtid_reset();
 	DBG("process %d", getpid());
 	/* Release urcu mutexes */
